@@ -7,7 +7,7 @@
 ;;
 ;; Emacs Lisp Archive Entry
 ;; Filename: emacs-wiki-publish.el
-;; Version: 2.68
+;; Version: 2.69
 ;; Keywords: hypermedia
 ;; Author: John Wiegley (johnw AT gnu DOT org)
 ;;         Alex Schroeder (alex AT gnu DOT org)
@@ -66,13 +66,15 @@
 ;;
 ;; Li Daobing (lidaobing AT gmail DOT com) helped the
 ;; `emacs-wiki-escape-url' function make better decoding decisions (10
-;; changed lines).
+;; changed lines).  He also helped fix an XHTML validation problem
+;; with alt attributes of IMG tags.
 ;;
 ;; Andrew J. Korty (ajk AT iu DOT edu) fixed a case where the current
 ;; project name is lost during publishing (3 changed lines).
 ;;
 ;; Trent Buck (fubarbaz AT bigpond DOT com) got rid of leading
-;; newlines between pre tags.
+;; newlines between pre tags.  He also suggested replacing ``'' with
+;; smart quotes.
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
@@ -181,6 +183,29 @@ particular charset"
   :type '(choice string symbol)
   :group 'emacs-wiki-publish)
 
+(defcustom emacs-wiki-xhtml-inline-css t
+  "If non-nil, generate XHTML-compliant output with respect to inline CSS.
+
+There are some tags like '<ul>' that are valid in HTML but not in
+XHTML.  If non-nil, this option causes CSS to be used in a way to
+emulate the effect of these tags.  If nil, just use the tag and
+don't worry about XHTML compatibility."
+  :type 'boolean
+  :group 'emacs-wiki-publish)
+
+(defcustom emacs-wiki-xhtml-space-singles t
+  "If non-nil, generate XHTML-compliant output with respect to single tags.
+
+In XHTML, tags that have no corresponding tag, like '<br>' must
+be represented as '<br />', with the extra space and forward
+slash.  This causes the document to not validate properly if you
+want HTML 4.0 compliant output.
+
+If non-nil, this option favors the XHTML way of doing things.  If
+nil, favor the HTML-friendly markup style."
+  :type 'boolean
+  :group 'emacs-wiki-publish)
+
 (defcustom emacs-wiki-charset-default "utf-8"
   "The default HTML meta charset to use if no translation is found in
 `emacs-wiki-coding-map'"
@@ -508,7 +533,9 @@ that verbatim is processed long before table is even seen."
    ["^#\\(\\S-+\\)\\s-*" 0 emacs-wiki-markup-anchor]
 
    ;; horizontal rule, or section separator
-   ["^----+" 0 "<hr />"]
+   `["^----+" 0 ,(if emacs-wiki-xhtml-space-singles
+                     "<hr />"
+                   "<hr>")]
 
    ;; expand macros
    [emacs-wiki-macro-regexp 0 emacs-wiki-macro-expand]
@@ -583,7 +610,13 @@ that verbatim is processed long before table is even seen."
    ["''''" 0 ""]
 
    ;; footnotes section is separated by a horizontal rule in HTML
-   ["^\\(\\* \\)?Footnotes:?\\s-*" 0 "</p><hr />\n<p>\n"]
+   `["^\\(\\* \\)?Footnotes:?\\s-*" 0
+     ,(concat "</p>"
+              (if emacs-wiki-xhtml-space-singles
+                  "<hr />"
+                "<hr>")
+              "\n<p>\n")]
+
    ;; footnote definition/reference (def if at beginning of line)
    ["\\[\\([1-9][0-9]*\\)\\]" 0 emacs-wiki-markup-footnote]
 
@@ -596,8 +629,9 @@ that verbatim is processed long before table is even seen."
     0
     "\\1<a href=\"mailto:\\2\">\\2</a>\\4")
 
-   ;; replace quotes, since most browsers don't understand `` and ''
-   ["\\(``\\|''\\)" 0 "\""]
+   ;; use smart quotes
+   ["\\(``\\)" 0 "&ldquo;"]
+   ["\\(''\\)" 0 "&rdquo;"]
 
    ;; insert the default publishing header
    (function
@@ -1266,11 +1300,14 @@ If CODING is used, use that coding system."
                (t
                 (format "%%%02x" ch)))) ; escape
             ;; Coerce a string to a list of chars.
-            (append (encode-coding-string
-                     str
-                     (or coding
-                         emacs-wiki-publish-url-coding-system
-                         'utf-8))
+            (append (if (fboundp 'encode-coding-string)
+                        (encode-coding-string
+                         str
+                         (or coding
+                             emacs-wiki-publish-url-coding-system
+                             'utf-8))
+                      ;; Do not transform if function not defined
+                      str)
                     nil)))))
 
 ;; we currently only do this on links. this means a stray '&' in an
@@ -1299,7 +1336,10 @@ misinterpreted if not escaped"
                             (t char))))
           (if (string= (char-charset char) "ascii")
               (setq len (length code)
-                    str (replace-match (concat "&#" code ";") nil nil str)
+                    str (concat (substring str 0 pos)
+                                "&#" code ";"
+                                (when (< pos (length str))
+                                  (substring str (1+ pos) nil)))
                     pos (+ 3 len pos))
 	    (cond ((string= (string (char-syntax char)) "w")
                    (setq pos (+ 1 pos)))
@@ -1311,8 +1351,10 @@ misinterpreted if not escaped"
                    (setq pos (+ 1 pos)))
 		  (t
                    (setq len (length code)
-                         str (replace-match (concat "&#" code ";")
-                                            nil nil str)
+                         str (concat (substring str 0 pos)
+                                     "&#" code ";"
+                                     (when (< pos (length str))
+                                       (substring str (1+ pos) nil)))
                          pos (+ 3 len pos))))))
         str))))
 
@@ -1448,7 +1490,10 @@ misinterpreted if not escaped"
    ((string= (match-string 1) "style")
     (set (make-local-variable 'emacs-wiki-style-sheet)
           (concat "<link rel=\"stylesheet\" type=\"text/css\" href=\""
-                  (match-string 2) "\" />")))
+                  (match-string 2)
+                  (if emacs-wiki-xhtml-space-singles
+                      "\" />"
+                    "\">"))))
    ((string= (match-string 1) "related")
     (set (make-local-variable 'emacs-wiki-current-page-related)
          (match-string 2))))
@@ -1526,9 +1571,13 @@ misinterpreted if not escaped"
     (goto-char beg)
     (setq end (copy-marker (1- end)))
     (insert "<table cellspacing=\"8\">")
-    (insert (format "<tr><td valign=\"top\"><strong>%d</strong></td>
-<td><p><a id=\"%d\" />" emacs-wiki-numbered-counter
-                         emacs-wiki-numbered-counter))
+    (insert (format
+             (concat "<tr><td valign=\"top\"><strong>%d</strong></td>"
+                     "<td><p><a id=\"%d\""
+                     (if emacs-wiki-xhtml-space-singles
+                         " />"
+                       ">"))
+             emacs-wiki-numbered-counter emacs-wiki-numbered-counter))
     (setq emacs-wiki-numbered-counter
           (1+ emacs-wiki-numbered-counter))
     (while (and (< (point) end)
@@ -1909,14 +1958,25 @@ Images used the <img> tag."
                 (if (string-match emacs-wiki-image-regexp name)
                     (if (string-match "^\\([^ ]+\\)\\s-+\\(.+\\)" name)
                         ;; [[image][image2 caption]]
-                        (format "<img src=\"%s\" alt=\"%s\" />"
+                        (format (concat
+                                 "<img src=\"%s\" alt=\"%s\""
+                                 (if emacs-wiki-xhtml-space-singles
+                                     " />"
+                                   ">"))
                                 (match-string 1 name)
-                                (emacs-wiki-escape-html-string (match-string 2 name)))
+                                (emacs-wiki-escape-html-string
+                                 (match-string 2 name)))
                       ;; [[image][image2]]
-                      (concat "<img src=\"" name "\" />"))
+                      (concat "<img src=\"" name "\" alt=\"\""
+                              (if emacs-wiki-xhtml-space-singles
+                                  " />"
+                                ">")))
                   ;; [[image][caption]]
                   (if (and url (string-match emacs-wiki-image-regexp url))
-                      (format "<img src=\"%s\" alt=\"%s\" />"
+                      (format (concat "<img src=\"%s\" alt=\"%s\""
+                                      (if emacs-wiki-xhtml-space-singles
+                                          " />"
+                                        ">"))
                               url (emacs-wiki-escape-html-string name))
                     ;; [[link][caption]]
                     (emacs-wiki-escape-html-string name)))))
@@ -1950,8 +2010,12 @@ Images used the <img> tag."
          open-tag close-tag mark-read-only loc multi-line)
     (cond
      ((string= leader "_")
-      (setq open-tag "<span style=\"text-decoration: underline\">"
-            close-tag "</span>"))
+      (setq open-tag (if emacs-wiki-xhtml-inline-css
+                         "<span style=\"text-decoration: underline;\">"
+                       "<u>")
+            close-tag (if emacs-wiki-xhtml-inline-css
+                          "</span>"
+                        "</u>")))
      ((string= leader "=")
       (setq open-tag "<code>" close-tag "</code>")
       (setq mark-read-only t))
@@ -2042,17 +2106,14 @@ Unicode browsers, this is usually unnecessary."
 The reason this function is so funky, is to prevent text properties
 like read-only from being inadvertently deleted."
   (if (null (match-string 2))
-      (let* ((ws (match-string 4))
-             (tag (if (>= (string-width ws) 6)
-                      "center"
-                    "blockquote")))
-        (unless (and (equal tag "blockquote")
+      (let ((ws (match-string 4)))
+        (unless (and (< (string-width ws) 6)
                      (save-excursion
                        (forward-line)
                        (or (eolp)
                            (looking-at "\\S-"))))
-          (emacs-wiki-surround-text (format "<%s>\n<p>\n%s" tag ws)
-                                    (format "\n</p>\n</%s>\n" tag)
+          (emacs-wiki-surround-text (concat "<blockquote>\n<p>\n" ws)
+                                    "\n</p>\n</blockquote>\n"
                                     'emacs-wiki-forward-paragraph)))
     (let ((str (match-string 2)))
       (cond
