@@ -11,7 +11,7 @@
 
 ;; Emacs Lisp Archive Entry
 ;; Filename: emacs-wiki.el
-;; Version: 2.68
+;; Version: 2.69
 ;; Keywords: hypermedia
 ;; Author: John Wiegley (johnw AT gnu DOT org)
 ;;         Alex Schroeder (alex AT gnu DOT org)
@@ -257,7 +257,7 @@
 ;; for `emacs-wiki-wiki-link-target' that helps deal with edge cases
 ;; in links like "[[#8]]" (10 changed lines).
 
-(defvar emacs-wiki-version "2.68"
+(defvar emacs-wiki-version "2.69"
   "The version of emacs-wiki currently loaded.")
 
 (require 'derived)
@@ -615,6 +615,14 @@ All wikis reside in the `emacs-wiki-directories'.
     (set (make-variable-buffer-local
           'pcomplete-parse-arguments-function)
          'emacs-wiki-current-word))
+  ;; Make fill not split up links
+  (when (boundp 'fill-nobreak-predicate)
+    (make-local-variable 'fill-nobreak-predicate)
+    ;; Work around annoying inconsistency in fill handling
+    (if (not (emacs-wiki-extreg-usable-p))
+        (setq fill-nobreak-predicate 'emacs-wiki-fill-nobreak-p)
+      (add-to-list 'fill-nobreak-predicate
+                   'emacs-wiki-fill-nobreak-p)))
   ;; avoid problems caused by emacs-wiki-url-protocols not getting
   ;; custom-set properly.
   (emacs-wiki-set-sym-and-url-regexp 'emacs-wiki-url-protocols
@@ -623,6 +631,11 @@ All wikis reside in the `emacs-wiki-directories'.
   ;; paragraph filling.
   (set (make-local-variable 'inhibit-point-motion-hooks) t))
 
+;; Make flyspell not mess with links
+(put 'emacs-wiki-mode
+     'flyspell-mode-predicate
+     'emacs-wiki-flyspell-p)
+
 (defsubst emacs-wiki-page-file (page &optional no-check-p)
   "Return a filename if PAGE exists within the current Wiki."
   (cdr (assoc page (emacs-wiki-file-alist no-check-p))))
@@ -674,6 +687,27 @@ If SUBSTITUTE is non-nil, use `replace-match'-style substitutions."
 
 ;;; Other functions
 
+(defun emacs-wiki-fill-nobreak-p ()
+  "Return nil if we should allow a fill to occur at point.
+Otherwise return non-nil.
+
+This is used to keep long extended links from being mangled by
+autofill."
+  (save-excursion
+    (save-match-data
+      (and (re-search-backward "\\[\\[\\|\\]\\]"
+                               (line-beginning-position) t)
+           (string= (or (match-string 0) "")
+                    "[[")))))
+
+(defun emacs-wiki-flyspell-p ()
+  "Return non-nil if we should allow spell-checking to occur at point.
+Otherwise return nil.
+
+This is used to keep links from being improperly colorized by flyspell."
+  (save-match-data
+    (null (emacs-wiki-link-at-point))))
+
 (defun emacs-wiki-directories-member (&optional directories)
   "Return non-nil if the current buffer is in `emacs-wiki-directories'."
   (unless (null buffer-file-name)
@@ -1200,14 +1234,32 @@ If OTHER-WINDOW is non-nil, open in a different window."
       (when entry
         (funcall (cadr entry) url)))))
 
+(defun emacs-wiki-get-current-backlink ()
+  "Return the backlink in the current page or nil if there is none."
+  (save-excursion
+    (save-match-data
+      (goto-char (point-min))
+      (when (looking-at "\\*\\*/.*\\*\\*\n")
+	(buffer-substring 
+	 (+ (point-min) 2) (- (match-end 0) 3))))))
+
+(defun emacs-wiki-insert-backlink (backlink &optional parent-links)
+  "Insert BACKLINK in the current page.
+If PARENT-LINKS is non-nil, insert it before BACKLINK."
+  (save-excursion
+    (goto-char (point-min))
+    (insert "**")
+    (when parent-links
+      (insert parent-links " "))
+    (insert "/ " backlink "**\n\n")))
+
 (defun emacs-wiki-visit-link
   (link-name &optional refresh-buffer other-window)
   "Visit the URL or link named by LINK-NAME.
 REFRESH-BUFFER is an optional buffer to refresh on saving the
 visited page.  This makes the bad link face in the linking buffer
 go away."
-  (let ((link (emacs-wiki-wiki-link-target link-name))
-        newbuf)
+  (let ((link (emacs-wiki-wiki-link-target link-name)))
     (if (emacs-wiki-wiki-url-p link)
         (emacs-wiki-browse-url link other-window)
       ;; The name list is current since the last time the buffer was
@@ -1219,50 +1271,52 @@ go away."
              (find-file-function (if other-window
                                      'find-file-other-window
                                    'find-file))
-	     ;; the existing backlink line in the buffer that contains the link
-	     (parent-backlink
-	      (if emacs-wiki-create-backlinks
-		  (save-match-data
-		    (goto-char (point-min))
-		    (if (looking-at "\*\*/.*\*\*\n")
-			(concat (buffer-substring (point-min) (- (match-end 0) 3)) " ")
-		      "**"))
-		""))
-
-	     ;; the name of the buffer that contains the link.     
-	     ;; check whether buffer-name is a WikiName, else make it one
-	     (parent-name (if (string-match (buffer-name) emacs-wiki-name-regexp)
-			      (buffer-name)
-			    (concat "[[" (buffer-name) "]]")))
-	     
-	     )
-        (setq newbuf
-              (if (null file)
-		  (funcall find-file-function
-			   (expand-file-name
-			    base
-			    (file-name-directory (buffer-file-name))))
-                (funcall find-file-function file))
-	      )
+	     ;; the name of the buffer that contains the link.  check
+	     ;; whether buffer-name is a WikiName, else make it one
+	     (parent-name (or (emacs-wiki-page-name)
+			      (concat "[[" (buffer-name) "]]")))
+	     (parent-allowed-backlink-p
+	      (not (string-match
+		    emacs-wiki-exclude-backlink-parent-regexp
+		    parent-name)))
+	     (parent-backlink (and parent-allowed-backlink-p
+				   (emacs-wiki-get-current-backlink)))
+	     (newbuf (if (null file)
+			 (funcall find-file-function
+				  (expand-file-name
+				   base
+				   (file-name-directory 
+				    (buffer-file-name))))
+		       (funcall find-file-function file)))
+	     (emacs-wiki-create-backlinks
+	      (and emacs-wiki-create-backlinks parent-allowed-backlink-p))
+	     (make-link-p (and emacs-wiki-create-backlinks
+			       ;; insert backlink only in new files
+			       (null file)
+			       (not (string-match
+				     emacs-wiki-exclude-backlink-regexp
+				     (emacs-wiki-page-name 
+				      (buffer-file-name newbuf)))))))
         (when tag
           (goto-char (point-min))
           (re-search-forward (concat "^\\.?#" tag) nil t))
         (when refresh-buffer
-	  (if emacs-wiki-create-backlinks
-	      ;; insert backlink only in new files
-	      (when (and (null file) (not (string-match emacs-wiki-exclude-backlink-regexp (buffer-name newbuf))))
-		(save-excursion
-		  (goto-char (point-min))
-		  (insert (concat parent-backlink "/ "parent-name "**\n\n")))))
-          (with-current-buffer newbuf
+	  (when make-link-p
+	    (emacs-wiki-insert-backlink parent-name parent-backlink))
+	  ;; todo: is with-current-buffer necessary here?
+	  (with-current-buffer newbuf
             (add-hook 'after-save-hook
                       'emacs-wiki-refresh-buffers-once t t)
-	    (if emacs-wiki-create-backlinks
-		;; save the new file to avoid another backlink if
-		;; the buffer is visited again before being saved
-		(when (and (null file) (not (string-match emacs-wiki-exclude-backlink-regexp (buffer-name newbuf))))
-	      (save-buffer)))
-))))))
+	    ;; save the new file to avoid another backlink if the
+	    ;; buffer is visited again before being saved
+	    (when (or make-link-p 
+		      (and (null file)
+			   (not parent-allowed-backlink-p)))
+	      ;; set modified status to t: pages with excluded
+	      ;; parents may not be modified, but we want to save
+	      ;; anyway 
+	      (set-buffer-modified-p t)
+ 	      (save-buffer))))))))
 
 (defun emacs-wiki-refresh-buffers-once ()
   "Refresh all the buffers once from after-save-hook.
